@@ -15,12 +15,12 @@
 import sys
 from os import path
 
-__version__  = '1.2'
-__released__ = '1.2'  # used when Sphinx builds its own docs
+__version__  = '1.2.1'
+__released__ = '1.2.1'  # used when Sphinx builds its own docs
 # version info for better programmatic use
 # possible values for 3rd element: 'alpha', 'beta', 'rc', 'final'
 # 'final' has 0 as the last element
-version_info = (1, 2, 0, 'final', 0)
+version_info = (1, 2, 1, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -80,5 +80,11 @@ def main(argv=sys.argv):
     return cmdline.main(argv)
 
 
+def make_main(argv=sys.argv):
+    """Sphinx build "make mode" entry."""
+    from sphinx import make_mode
+    return make_mode.run_make_mode(argv[2:])
+
+
 if __name__ == '__main__':
     sys.exit(main(sys.argv))
