@@ -597,7 +597,9 @@ class StandaloneHTMLBuilder(Builder):
         if self.config.html_logo:
             logobase = path.basename(self.config.html_logo)
             logotarget = path.join(self.outdir, '_static', logobase)
-            if not path.isfile(logotarget):
+            if not path.isfile(logobase):
+                self.warn('logo file %r does not exist' % logobase)
+            elif not path.isfile(logotarget):
                 copyfile(path.join(self.confdir, self.config.html_logo),
                          logotarget)
         if self.config.html_favicon:
@@ -618,6 +620,7 @@ class StandaloneHTMLBuilder(Builder):
                 self.warn('html_extra_path entry %r does not exist' % entry)
                 continue
             copy_static_entry(entry, self.outdir, self)
+        self.info('done')
 
     def write_buildinfo(self):
         # write build info file
