@@ -5,7 +5,7 @@
 
     Build configuration file handling.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,7 +17,7 @@ from os import path
 from sphinx.errors import ConfigError
 from sphinx.locale import l_
 from sphinx.util.osutil import make_filename
-from sphinx.util.pycompat import bytes, b, convert_with_2to3
+from sphinx.util.pycompat import bytes, b, execfile_
 
 nonascii_re = re.compile(b(r'[\x80-\xff]'))
 
@@ -72,7 +72,7 @@ class Config(object):
         primary_domain = ('py', 'env'),
         needs_sphinx = (None, None),
         nitpicky = (False, 'env'),
-        nitpick_ignore = ([], 'env'),
+        nitpick_ignore = ([], 'html'),
 
         # HTML options
         html_theme = ('default', 'html'),
@@ -86,6 +86,7 @@ class Config(object):
         html_logo = (None, 'html'),
         html_favicon = (None, 'html'),
         html_static_path = ([], 'html'),
+        html_extra_path = ([], 'html'),
         # the real default is locale-dependent
         html_last_updated_fmt = (None, 'html'),
         html_use_smartypants = (True, 'html'),
@@ -110,6 +111,7 @@ class Config(object):
         html_secnumber_suffix = ('. ', 'html'),
         html_search_language = (None, 'html'),
         html_search_options = ({}, 'html'),
+        html_search_scorer = ('', None),
 
         # HTML help only options
         htmlhelp_basename = (lambda self: make_filename(self.project), None),
@@ -123,6 +125,7 @@ class Config(object):
         # Epub options
         epub_basename = (lambda self: make_filename(self.project), None),
         epub_theme = ('epub', 'html'),
+        epub_theme_options = ({}, 'html'),
         epub_title = (lambda self: self.html_title, 'html'),
         epub_author = ('unknown', 'html'),
         epub_language = (lambda self: self.language or 'en', 'html'),
@@ -132,14 +135,24 @@ class Config(object):
         epub_scheme = ('unknown', 'html'),
         epub_uid = ('unknown', 'env'),
         epub_cover = ((), 'env'),
+        epub_guide = ((), 'env'),
         epub_pre_files = ([], 'env'),
         epub_post_files = ([], 'env'),
         epub_exclude_files = ([], 'env'),
         epub_tocdepth = (3, 'env'),
         epub_tocdup = (True, 'env'),
+        epub_tocscope = ('default', 'env'),
+        epub_fix_images = (False, 'env'),
+        epub_max_image_width = (0, 'env'),
+        epub_show_urls = ('inline', 'html'),
+        epub_use_index = (lambda self: self.html_use_index, 'html'),
 
         # LaTeX options
-        latex_documents = ([], None),
+        latex_documents = (lambda self: [(self.master_doc,
+                                          make_filename(self.project) + '.tex',
+                                          self.project,
+                                          '', 'manual')],
+                           None),
         latex_logo = (None, None),
         latex_appendices = ([], None),
         latex_use_parts = (False, None),
@@ -162,29 +175,47 @@ class Config(object):
         text_newlines = ('unix', 'env'),
 
         # manpage options
-        man_pages = ([], None),
+        man_pages = (lambda self: [(self.master_doc,
+                                    make_filename(self.project).lower(),
+                                    '%s %s' % (self.project, self.release),
+                                    [], 1)],
+                     None),
         man_show_urls = (False, None),
 
         # Texinfo options
-        texinfo_documents = ([], None),
+        texinfo_documents = (lambda self: [(self.master_doc,
+                                            make_filename(self.project).lower(),
+                                            self.project, '',
+                                            make_filename(self.project),
+                                            'The %s reference manual.' %
+                                            make_filename(self.project),
+                                            'Python')],
+                             None),
         texinfo_appendices = ([], None),
         texinfo_elements = ({}, None),
         texinfo_domain_indices = (True, None),
         texinfo_show_urls = ('footnote', None),
+        texinfo_no_detailmenu = (False, None),
 
         # linkcheck options
         linkcheck_ignore = ([], None),
         linkcheck_timeout = (None, None),
         linkcheck_workers = (5, None),
+        linkcheck_anchors = (True, None),
 
         # gettext options
         gettext_compact = (True, 'gettext'),
+
+        # XML options
+        xml_pretty = (True, 'env'),
     )
 
     def __init__(self, dirname, filename, overrides, tags):
         self.overrides = overrides
         self.values = Config.config_values.copy()
         config = {}
+        if "extensions" in overrides:
+            config["extensions"] = overrides["extensions"]
         if dirname is not None:
             config_file = path.join(dirname, filename)
             config['__file__'] = config_file
@@ -194,25 +225,8 @@ class Config(object):
                 # we promise to have the config dir as current dir while the
                 # config file is executed
                 os.chdir(dirname)
-                # get config source
-                f = open(config_file, 'rb')
                 try:
-                    source = f.read()
-                finally:
-                    f.close()
-                try:
-                    # compile to a code object, handle syntax errors
-                    try:
-                        code = compile(source, config_file, 'exec')
-                    except SyntaxError:
-                        if convert_with_2to3:
-                            # maybe the file uses 2.x syntax; try to refactor to
-                            # 3.x syntax using 2to3
-                            source = convert_with_2to3(config_file)
-                            code = compile(source, config_file, 'exec')
-                        else:
-                            raise
-                    exec code in config
+                    execfile_(filename, config)
                 except SyntaxError, err:
                     raise ConfigError(CONFIG_SYNTAX_ERROR % err)
             finally:
