@@ -177,9 +177,6 @@ class BuildEnvironment:
         # this is to invalidate old pickles
         self.version = ENV_VERSION
 
-        # make this a set for faster testing
-        self._nitpick_ignore = set(self.config.nitpick_ignore)
-
         # All "docnames" here are /-separated and relative and exclude
         # the source suffix.
 
@@ -440,6 +437,9 @@ class BuildEnvironment:
         self.find_files(config)
         self.config = config
 
+        # this cache also needs to be updated every time
+        self._nitpick_ignore = set(self.config.nitpick_ignore)
+
         added, changed, removed = self.get_outdated_files(config_changed)
 
         # allow user intervention as well
@@ -614,11 +614,8 @@ class BuildEnvironment:
         pub.process_programmatic_settings(None, self.settings, None)
         pub.set_source(None, src_path.encode(fs_encoding))
         pub.set_destination(None, None)
-        try:
-            pub.publish()
-            doctree = pub.document
-        except UnicodeError, err:
-            raise SphinxError(str(err))
+        pub.publish()
+        doctree = pub.document
 
         # post-processing
         self.filter_messages(doctree)
@@ -1385,6 +1382,9 @@ class BuildEnvironment:
                 dtype = domain and '%s:%s' % (domain.name, typ) or typ
                 if (dtype, target) in self._nitpick_ignore:
                     warn = False
+                # for "std" types also try without domain name
+                if domain.name == 'std' and (typ, target) in self._nitpick_ignore:
+                    warn = False
         if not warn:
             return
         if domain and typ in domain.dangling_warnings:
