@@ -6,14 +6,14 @@
     Allow graphviz-formatted graphs to be included in Sphinx-generated
     documents inline.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+import codecs
 import posixpath
 from os import path
-from math import ceil
 from subprocess import Popen, PIPE
 try:
     from hashlib import sha1 as sha
@@ -24,12 +24,12 @@ from docutils import nodes
 from docutils.parsers.rst import directives
 
 from sphinx.errors import SphinxError
-from sphinx.util.osutil import ensuredir, ENOENT, EPIPE
+from sphinx.locale import _
+from sphinx.util.osutil import ensuredir, ENOENT, EPIPE, EINVAL
 from sphinx.util.compat import Directive
 
 
 mapname_re = re.compile(r'<map id="(.*?)"')
-svg_dim_re = re.compile(r'<svg\swidth="(\d+)pt"\sheight="(\d+)pt"', re.M)
 
 
 class GraphvizError(SphinxError):
@@ -46,23 +46,48 @@ class Graphviz(Directive):
     """
     has_content = True
     required_arguments = 0
-    optional_arguments = 0
+    optional_arguments = 1
     final_argument_whitespace = False
     option_spec = {
         'alt': directives.unchanged,
+        'inline': directives.flag,
+        'caption': directives.unchanged,
     }
 
     def run(self):
-        dotcode = '\n'.join(self.content)
-        if not dotcode.strip():
-            return [self.state_machine.reporter.warning(
-                'Ignoring "graphviz" directive without content.',
-                line=self.lineno)]
+        if self.arguments:
+            document = self.state.document
+            if self.content:
+                return [document.reporter.warning(
+                    'Graphviz directive cannot have both content and '
+                    'a filename argument', line=self.lineno)]
+            env = self.state.document.settings.env
+            rel_filename, filename = env.relfn2path(self.arguments[0])
+            env.note_dependency(rel_filename)
+            try:
+                fp = codecs.open(filename, 'r', 'utf-8')
+                try:
+                    dotcode = fp.read()
+                finally:
+                    fp.close()
+            except (IOError, OSError):
+                return [document.reporter.warning(
+                    'External Graphviz file %r not found or reading '
+                    'it failed' % filename, line=self.lineno)]
+        else:
+            dotcode = '\n'.join(self.content)
+            if not dotcode.strip():
+                return [self.state_machine.reporter.warning(
+                    'Ignoring "graphviz" directive without content.',
+                    line=self.lineno)]
         node = graphviz()
         node['code'] = dotcode
         node['options'] = []
         if 'alt' in self.options:
             node['alt'] = self.options['alt']
+        if 'caption' in self.options:
+            node['caption'] = self.options['caption']
+        node['inline'] = 'inline' in self.options
         return [node]
 
 
@@ -76,6 +101,8 @@ class GraphvizSimple(Directive):
     final_argument_whitespace = False
     option_spec = {
         'alt': directives.unchanged,
+        'inline': directives.flag,
+        'caption': directives.unchanged,
     }
 
     def run(self):
@@ -85,15 +112,19 @@ class GraphvizSimple(Directive):
         node['options'] = []
         if 'alt' in self.options:
             node['alt'] = self.options['alt']
+        if 'caption' in self.options:
+            node['caption'] = self.options['caption']
+        node['inline'] = 'inline' in self.options
         return [node]
 
 
 def render_dot(self, code, options, format, prefix='graphviz'):
-    """
-    Render graphviz code into a PNG or PDF output file.
-    """
-    hashkey = code.encode('utf-8') + str(options) + \
+    """Render graphviz code into a PNG or PDF output file."""
+    hashkey = (code + str(options) + \
+              str(self.builder.config.graphviz_dot) + \
               str(self.builder.config.graphviz_dot_args)
+              ).encode('utf-8')
+
     fname = '%s-%s.%s' % (prefix, sha(hashkey).hexdigest(), format)
     if hasattr(self.builder, 'imgpath'):
         # HTML
@@ -137,8 +168,8 @@ def render_dot(self, code, options, format, prefix='graphviz'):
         # Graphviz may close standard input when an error occurs,
         # resulting in a broken pipe on communicate()
         stdout, stderr = p.communicate(code)
-    except OSError, err:
-        if err.errno != EPIPE:
+    except (OSError, IOError), err:
+        if err.errno not in (EPIPE, EINVAL):
             raise
         # in this case, read the standard output and standard error streams
         # directly, to get the error message(s)
@@ -147,40 +178,12 @@ def render_dot(self, code, options, format, prefix='graphviz'):
     if p.returncode != 0:
         raise GraphvizError('dot exited with error:\n[stderr]\n%s\n'
                             '[stdout]\n%s' % (stderr, stdout))
+    if not path.isfile(outfn):
+        raise GraphvizError('dot did not produce an output file:\n[stderr]\n%s\n'
+                            '[stdout]\n%s' % (stderr, stdout))
     return relfn, outfn
 
 
-def get_svg_tag(svgref, svgfile, imgcls=None):
-    # Webkit can't figure out svg dimensions when using object tag
-    # so we need to get it from the svg file
-    fp = open(svgfile, 'r')
-    try:
-        for line in fp:
-            match = svg_dim_re.match(line)
-            if match:
-                dimensions = match.groups()
-                break
-        else:
-            dimensions = None
-    finally:
-        fp.close()
-
-    # We need this hack to make WebKit show our object tag properly
-    def pt2px(x):
-        return int(ceil((96.0/72.0) * float(x)))
-
-    if dimensions:
-        style = ' width="%s" height="%s"' % tuple(map(pt2px, dimensions))
-    else:
-        style = ''
-
-    # The object tag works fine on Firefox and WebKit
-    # Besides it's a hack, this strategy does not mess with templates.
-    imgcss = imgcls and ' class="%s"' % imgcls or ''
-    return '<object type="image/svg+xml" data="%s"%s%s/>\n' % \
-           (svgref, imgcss, style)
-
-
 def render_dot_html(self, node, code, options, prefix='graphviz',
                     imgcls=None, alt=None):
     format = self.builder.config.graphviz_output_format
@@ -193,14 +196,21 @@ def render_dot_html(self, node, code, options, prefix='graphviz',
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
 
-    self.body.append(self.starttag(node, 'p', CLASS='graphviz'))
+    inline = node.get('inline', False)
+    if inline:
+        wrapper = 'span'
+    else:
+        wrapper = 'p'
+
+    self.body.append(self.starttag(node, wrapper, CLASS='graphviz'))
     if fname is None:
         self.body.append(self.encode(code))
     else:
         if alt is None:
             alt = node.get('alt', self.encode(code).strip())
+        imgcss = imgcls and 'class="%s"' % imgcls or ''
         if format == 'svg':
-            svgtag = get_svg_tag(fname, outfn, imgcls)
+            svgtag = '<img src="%s" alt="%s" %s/>\n' % (fname, alt, imgcss)
             self.body.append(svgtag)
         else:
             mapfile = open(outfn + '.map', 'rb')
@@ -208,19 +218,21 @@ def render_dot_html(self, node, code, options, prefix='graphviz',
                 imgmap = mapfile.readlines()
             finally:
                 mapfile.close()
-            imgcss = imgcls and 'class="%s"' % imgcls or ''
             if len(imgmap) == 2:
                 # nothing in image map (the lines are <map> and </map>)
                 self.body.append('<img src="%s" alt="%s" %s/>\n' %
                                  (fname, alt, imgcss))
             else:
                 # has a map: get the name of the map and connect the parts
-                mapname = mapname_re.match(imgmap[0]).group(1)
+                mapname = mapname_re.match(imgmap[0].decode('utf-8')).group(1)
                 self.body.append('<img src="%s" alt="%s" usemap="#%s" %s/>\n' %
                                  (fname, alt, mapname, imgcss))
-                self.body.extend(imgmap)
+                self.body.extend([item.decode('utf-8') for item in imgmap])
+        if node.get('caption') and not inline:
+            self.body.append('</p>\n<p class="caption">')
+            self.body.append(self.encode(node['caption']))
 
-    self.body.append('</p>\n')
+    self.body.append('</%s>\n' % wrapper)
     raise nodes.SkipNode
 
 
@@ -235,18 +247,74 @@ def render_dot_latex(self, node, code, options, prefix='graphviz'):
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
 
+    inline = node.get('inline', False)
+    if inline:
+        para_separator = ''
+    else:
+        para_separator = '\n'
+
     if fname is not None:
-        self.body.append('\\includegraphics{%s}' % fname)
+        caption = node.get('caption')
+        # XXX add ids from previous target node
+        if caption and not inline:
+            self.body.append('\n\\begin{figure}[h!]')
+            self.body.append('\n\\begin{center}')
+            self.body.append('\n\\caption{%s}' % self.encode(caption))
+            self.body.append('\n\\includegraphics{%s}' % fname)
+            self.body.append('\n\\end{center}')
+            self.body.append('\n\\end{figure}\n')
+        else:
+            self.body.append('%s\\includegraphics{%s}%s' %
+                             (para_separator, fname, para_separator))
     raise nodes.SkipNode
 
 
 def latex_visit_graphviz(self, node):
     render_dot_latex(self, node, node['code'], node['options'])
 
+
+def render_dot_texinfo(self, node, code, options, prefix='graphviz'):
+    try:
+        fname, outfn = render_dot(self, code, options, 'png', prefix)
+    except GraphvizError, exc:
+        self.builder.warn('dot code %r: ' % code + str(exc))
+        raise nodes.SkipNode
+    if fname is not None:
+        self.body.append('\n\n@float\n')
+        caption = node.get('caption')
+        if caption:
+            self.body.append('@caption{%s}\n' % self.escape_arg(caption))
+        self.body.append('@image{%s,,,[graphviz],png}\n'
+                         '@end float\n\n' % fname[:-4])
+    raise nodes.SkipNode
+
+def texinfo_visit_graphviz(self, node):
+    render_dot_texinfo(self, node, node['code'], node['options'])
+
+
+def text_visit_graphviz(self, node):
+    if 'alt' in node.attributes:
+        self.add_text(_('[graph: %s]') % node['alt'])
+    else:
+        self.add_text(_('[graph]'))
+    raise nodes.SkipNode
+
+
+def man_visit_graphviz(self, node):
+    if 'alt' in node.attributes:
+        self.body.append(_('[graph: %s]') % node['alt'])
+    else:
+        self.body.append(_('[graph]'))
+    raise nodes.SkipNode
+
+
 def setup(app):
     app.add_node(graphviz,
                  html=(html_visit_graphviz, None),
-                 latex=(latex_visit_graphviz, None))
+                 latex=(latex_visit_graphviz, None),
+                 texinfo=(texinfo_visit_graphviz, None),
+                 text=(text_visit_graphviz, None),
+                 man=(man_visit_graphviz, None))
     app.add_directive('graphviz', Graphviz)
     app.add_directive('graph', GraphvizSimple)
     app.add_directive('digraph', GraphvizSimple)
