@@ -5,12 +5,11 @@
 
     Highlight code blocks using Pygments.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
-import cgi
 import re
 import textwrap
 
@@ -20,7 +19,9 @@ except ImportError:
     # parser is not available on Jython
     parser = None
 
-from sphinx.util.texescape import tex_hl_escape_map_old, tex_hl_escape_map_new
+from sphinx.util.pycompat import htmlescape
+from sphinx.util.texescape import tex_hl_escape_map_new
+from sphinx.ext import doctest
 
 try:
     import pygments
@@ -30,41 +31,19 @@ try:
     from pygments.lexers import get_lexer_by_name, guess_lexer
     from pygments.formatters import HtmlFormatter, LatexFormatter
     from pygments.filters import ErrorToken
-    from pygments.style import Style
     from pygments.styles import get_style_by_name
-    from pygments.styles.friendly import FriendlyStyle
-    from pygments.token import Generic, Comment, Number
     from pygments.util import ClassNotFound
+    from sphinx.pygments_styles import SphinxStyle, NoneStyle
 except ImportError:
     pygments = None
     lexers = None
     HtmlFormatter = LatexFormatter = None
 else:
-    class SphinxStyle(Style):
-        """
-        Like friendly, but a bit darker to enhance contrast on the green
-        background.
-        """
-
-        background_color = '#eeffcc'
-        default_style = ''
-
-        styles = FriendlyStyle.styles
-        styles.update({
-            Generic.Output: '#333',
-            Comment: 'italic #408090',
-            Number: '#208050',
-        })
-
-    class NoneStyle(Style):
-        """Style without any styling."""
 
     lexers = dict(
         none = TextLexer(),
         python = PythonLexer(),
         pycon = PythonConsoleLexer(),
-        # the python3 option exists as of Pygments 1.0,
-        # but it doesn't do any harm in previous versions
         pycon3 = PythonConsoleLexer(python3=True),
         rest = RstLexer(),
         c = CLexer(),
@@ -73,19 +52,17 @@ else:
         _lexer.add_filter('raiseonerror')
 
 
-escape_hl_chars = {ord(u'@'): u'@PYGZat[]',
-                   ord(u'['): u'@PYGZlb[]',
-                   ord(u']'): u'@PYGZrb[]'}
+escape_hl_chars = {ord(u'\\'): u'\\PYGZbs{}',
+                   ord(u'{'): u'\\PYGZob{}',
+                   ord(u'}'): u'\\PYGZcb{}'}
 
 # used if Pygments is not available
 _LATEX_STYLES = r'''
-\newcommand\PYGZat{@}
-\newcommand\PYGZlb{[}
-\newcommand\PYGZrb{]}
+\newcommand\PYGZbs{\char`\\}
+\newcommand\PYGZob{\char`\{}
+\newcommand\PYGZcb{\char`\}}
 '''
 
-doctestopt_re = re.compile(r'#\s*doctest:.+$', re.MULTILINE)
-
 parsing_exceptions = (SyntaxError, UnicodeEncodeError)
 if sys.version_info < (2, 5):
     # Python <= 2.4 raises MemoryError when parsing an
@@ -115,24 +92,26 @@ class PygmentsBridge(object):
         else:
             style = get_style_by_name(stylename)
         self.trim_doctest_flags = trim_doctest_flags
+        self.formatter_args = {'style' : style}
         if dest == 'html':
-            self.fmter = {False: self.html_formatter(style=style),
-                          True: self.html_formatter(style=style, linenos=True)}
+            self.formatter = self.html_formatter
         else:
-            self.fmter = {False: self.latex_formatter(style=style,
-                                                      commandprefix='PYG'),
-                          True: self.latex_formatter(style=style, linenos=True,
-                                                     commandprefix='PYG')}
+            self.formatter = self.latex_formatter
+            self.formatter_args['commandprefix'] = 'PYG'
+
+    def get_formatter(self, **kwargs):
+        kwargs.update(self.formatter_args)
+        return self.formatter(**kwargs)
 
     def unhighlighted(self, source):
         if self.dest == 'html':
-            return '<pre>' + cgi.escape(source) + '</pre>\n'
+            return '<pre>' + htmlescape(source) + '</pre>\n'
         else:
             # first, escape highlighting characters like Pygments does
             source = source.translate(escape_hl_chars)
             # then, escape all characters nonrepresentable in LaTeX
-            source = source.translate(tex_hl_escape_map_old)
-            return '\\begin{Verbatim}[commandchars=@\\[\\]]\n' + \
+            source = source.translate(tex_hl_escape_map_new)
+            return '\\begin{Verbatim}[commandchars=\\\\\\{\\}]\n' + \
                    source + '\\end{Verbatim}\n'
 
     def try_parse(self, src):
@@ -156,7 +135,7 @@ class PygmentsBridge(object):
         if sys.version_info >= (2, 5):
             src = 'from __future__ import with_statement\n' + src
 
-        if isinstance(src, unicode):
+        if sys.version_info < (3, 0) and isinstance(src, unicode):
             # Non-ASCII chars will only occur in string literals
             # and comments.  If we wanted to give them to the parser
             # correctly, we'd have to find out the correct source
@@ -164,6 +143,12 @@ class PygmentsBridge(object):
             # just replace all non-ASCII characters.
             src = src.encode('ascii', 'replace')
 
+        if (3, 0) <= sys.version_info < (3, 2):
+            # Python 3.1 can't process '\r' as linesep.
+            # `parser.suite("print('hello')\r\n")` cause error.
+            if '\r\n' in src:
+                src = src.replace('\r\n', '\n')
+
         if parser is None:
             return True
 
@@ -174,8 +159,8 @@ class PygmentsBridge(object):
         else:
             return True
 
-    def highlight_block(self, source, lang, linenos=False, warn=None):
-        if isinstance(source, str):
+    def highlight_block(self, source, lang, warn=None, force=False, **kwargs):
+        if not isinstance(source, unicode):
             source = source.decode()
         if not pygments:
             return self.unhighlighted(source)
@@ -185,12 +170,14 @@ class PygmentsBridge(object):
             if source.startswith('>>>'):
                 # interactive session
                 lexer = lexers['pycon']
-            else:
+            elif not force:
                 # maybe Python -- try parsing it
                 if self.try_parse(source):
                     lexer = lexers['python']
                 else:
-                    return self.unhighlighted(source)
+                    lexer = lexers['none']
+            else:
+                lexer = lexers['python']
         elif lang in ('python3', 'py3') and source.startswith('>>>'):
             # for py3, recognize interactive sessions, but do not try parsing...
             lexer = lexers['pycon3']
@@ -198,7 +185,7 @@ class PygmentsBridge(object):
             try:
                 lexer = guess_lexer(source)
             except Exception:
-                return self.unhighlighted(source)
+                lexer = lexers['none']
         else:
             if lang in lexers:
                 lexer = lexers[lang]
@@ -207,8 +194,8 @@ class PygmentsBridge(object):
                     lexer = lexers[lang] = get_lexer_by_name(lang)
                 except ClassNotFound:
                     if warn:
-                        warn('Pygments lexer name %s is not known' % lang)
-                        return self.unhighlighted(source)
+                        warn('Pygments lexer name %r is not known' % lang)
+                        lexer = lexers['none']
                     else:
                         raise
                 else:
@@ -216,22 +203,23 @@ class PygmentsBridge(object):
 
         # trim doctest options if wanted
         if isinstance(lexer, PythonConsoleLexer) and self.trim_doctest_flags:
-            source = doctestopt_re.sub('', source)
+            source = doctest.blankline_re.sub('', source)
+            source = doctest.doctestopt_re.sub('', source)
 
         # highlight via Pygments
+        formatter = self.get_formatter(**kwargs)
         try:
-            if self.dest == 'html':
-                return highlight(source, lexer, self.fmter[bool(linenos)])
-            else:
-                hlsource = highlight(source, lexer, self.fmter[bool(linenos)])
-                if hlsource.startswith(r'\begin{Verbatim}[commandchars=\\\{\}'):
-                    # Pygments >= 1.2
-                    return hlsource.translate(tex_hl_escape_map_new)
-                return hlsource.translate(tex_hl_escape_map_old)
+            hlsource = highlight(source, lexer, formatter)
         except ErrorToken:
             # this is most probably not the selected language,
             # so let it pass unhighlighted
-            return self.unhighlighted(source)
+            hlsource = highlight(source, lexers['none'], formatter)
+        if self.dest == 'html':
+            return hlsource
+        else:
+            if not isinstance(hlsource, unicode):  # Py2 / Pygments < 1.6
+                hlsource = hlsource.decode()
+            return hlsource.translate(tex_hl_escape_map_new)
 
     def get_stylesheet(self):
         if not pygments:
@@ -239,11 +227,8 @@ class PygmentsBridge(object):
                 return _LATEX_STYLES
             # no HTML styles needed
             return ''
+        formatter = self.get_formatter()
         if self.dest == 'html':
-            return self.fmter[0].get_style_defs('.highlight')
+            return formatter.get_style_defs('.highlight')
         else:
-            styledefs = self.fmter[0].get_style_defs()
-            # workaround for Pygments < 0.12
-            if styledefs.startswith('\\newcommand\\at{@}'):
-                styledefs += _LATEX_STYLES
-            return styledefs
+            return formatter.get_style_defs()
